(ns com.aurellem.gb.experience
  (:use (com.aurellem.gb gb-driver util constants))
  (:import [com.aurellem.gb.gb_driver SaveState]))

(def experience-pokemon-1 0xD178)

(defn experience-start-address [poke-num]
  (+ experience-pokemon-1
     (* pokemon-record-width poke-num)))

(def experience-record-width 13)

(defn read-experience
  ([^SaveState state poke-num]
     (let [start (experience-start-address poke-num)
           [exp-h
            exp-m
            exp-l
            hp-exp-h
            hp-exp-l
            attack-exp-h
            attack-exp-l
            defense-exp-h
            defense-exp-l
            speed-exp-h
            speed-exp-l
            special-exp-h
            special-exp-l]
           (subvec (vec (memory state))
                   start (+ experience-record-width start))
           glue-bytes (fn [l h]
                        (+ l (bit-shift-left h 8)))]
       {:main-exp (+ (glue-bytes exp-l exp-m)
                     (bit-shift-left exp-h 16))
        :hp-exp (glue-bytes hp-exp-l hp-exp-h)
        :attack-exp (glue-bytes attack-exp-l attack-exp-h)
        :defense-exp (glue-bytes defense-exp-l defense-exp-h)
        :speed-exp (glue-bytes speed-exp-l speed-exp-h)
        :special-exp (glue-bytes special-exp-l special-exp-h)}))
  ([poke-num]
     (read-experience @current-state poke-num)))

(defn give-experience
  ([^SaveState state poke-num exp]
     (let [exp* (merge (read-experience state poke-num)
                       exp)

           disect-bytes
           (fn [exp]
             [(bit-shift-right
               (bit-and exp 0xFF00) 8)
              (bit-and exp 0xFF)])

           raw-exp-data
           (flatten
            [(bit-shift-right (bit-and (:main-exp exp*) 0xFF0000) 16)
             (disect-bytes (:main-exp exp*))
             (disect-bytes (:hp-exp exp*))
             (disect-bytes (:attack-exp exp*))
             (disect-bytes (:defense-exp exp*))
             (disect-bytes (:speed-exp exp*))
             (disect-bytes (:special-exp exp*))])]
       (set-memory-range state 
                         (experience-start-address poke-num)
                         raw-exp-data)))
  ([poke-num exp]
     (give-experience @current-state poke-num exp)))
